/**
 * FLL - Level 2
 *
 * Project: Program
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Defines common data to be used for/by project program.
 *
 * This is auto-included by program.h and should not need to be explicitly included.
 */
#ifndef _FLL_program_print_h
#define _FLL_program_print_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Print Featureless Linux Library copyright.
 *
 * @param print
 *   The output structure to print to.
 *   The print.verbosity is used to determine how much detail is printed (except for when _di_detailed_copyright_ is set).
 * @param year_author
 *   The year and author to be printed with the copyright symbol line.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_copyright_
  extern f_status_t fll_program_print_copyright(fl_print_t * const print, const f_string_static_t year_author);
#endif // _di_fll_program_print_copyright_

/**
 * Print an error message when a file is not provided.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_missing_file_
  extern f_status_t fll_program_print_error_missing_file(fl_print_t * const print);
#endif // _di_fll_program_print_error_missing_file_

/**
 * Print an error message a variable not being defined or the variable has a length is 0.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 * @param variable
 *   The name of the variable that is not defined or has a length of 0.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_missing_variable_not_zero_
  extern f_status_t fll_program_print_error_missing_variable_not_zero(fl_print_t * const print, const f_string_static_t variable);
#endif // _di_fll_program_print_error_missing_variable_not_zero_

/**
 * Print an error message about two parameters being required to be specified the same number of times.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 * @param symbol_1
 *   The symbol string prepended to the first parameter.
 *   This is usually f_console_symbol_long_normal_s.
 * @param symbol_2
 *   The symbol string prepended to the second parameter.
 *   This is usually f_console_symbol_long_normal_s.
 * @param name_1
 *   The first parameter name.
 * @param name_2
 *   The second parameter name.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_parameter_both_specified_same_amount_
  extern f_status_t fll_program_print_error_parameter_both_specified_same_amount(fl_print_t * const print, const f_string_static_t symbol_1, const f_string_static_t symbol_2, const f_string_static_t name_1, const f_string_static_t name_2);
#endif // _di_fll_program_print_error_parameter_both_specified_same_amount_

/**
 * Print an error message about two parameters being required to be specified the same number of times when not specifying the third parameter.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 * @param symbol_1
 *   The symbol string prepended to the first parameter.
 *   This is usually f_console_symbol_long_normal_s.
 * @param symbol_2
 *   The symbol string prepended to the second parameter.
 *   This is usually f_console_symbol_long_normal_s.
 * @param symbol_3
 *   The symbol string prepended to the third parameter.
 *   This is usually f_console_symbol_long_normal_s.
 * @param name_1
 *   The first parameter name.
 * @param name_2
 *   The second parameter name.
 * @param name_3
 *   The third parameter name, representing the "without" parameter.
 *   The first and second parameter must be specified the same amount of times without this parameter.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_parameter_both_specified_same_amount_without_
  extern f_status_t fll_program_print_error_parameter_both_specified_same_amount_without(fl_print_t * const print, const f_string_static_t symbol_1, const f_string_static_t symbol_2, const f_string_static_t symbol_3, const f_string_static_t name_1, const f_string_static_t name_2, const f_string_static_t name_3);
#endif // _di_fll_program_print_error_parameter_both_specified_same_amount_without_

/**
 * Print message about two parameters not being allowed to be used together.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 * @param symbol_1
 *   The symbol string prepended to the first parameter.
 *   This is usually f_console_symbol_long_normal_s.
 * @param symbol_2
 *   The symbol string prepended to the second parameter.
 *   This is usually f_console_symbol_long_normal_s.
 * @param name_1
 *   The first parameter name.
 * @param name_2
 *   The second parameter name.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_parameter_cannot_use_with_
  extern f_status_t fll_program_print_error_parameter_cannot_use_with(fl_print_t * const print, const f_string_static_t symbol_1, const f_string_static_t symbol_2, const f_string_static_t name_1, const f_string_static_t name_2);
#endif // _di_fll_program_print_error_parameter_cannot_use_with_

/**
 * Print message about two parameters not being allowed to be used together when a third is not used.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 * @param symbol_1
 *   The symbol string prepended to the first parameter.
 *   This is usually f_console_symbol_long_normal_s.
 * @param symbol_2
 *   The symbol string prepended to the second parameter.
 *   This is usually f_console_symbol_long_normal_s.
 * @param symbol_3
 *   The symbol string prepended to the second parameter.
 *   This is usually f_console_symbol_long_normal_s.
 * @param name_1
 *   The first parameter name (the parameter that cannot be used with the second parameter).
 * @param name_2
 *   The second parameter name.
 * @param name_3
 *   The third parameter name (the without parameter).
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_parameter_cannot_use_with_without_
  extern f_status_t fll_program_print_error_parameter_cannot_use_with_without(fl_print_t * const print, const f_string_static_t symbol_1, const f_string_static_t symbol_2, const f_string_static_t symbol_3, const f_string_static_t name_1, const f_string_static_t name_2, const f_string_static_t name_3);
#endif // _di_fll_program_print_error_parameter_cannot_use_with_without_

/**
 * Print message about one parameter not being allowed to be used when processing a pipe.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 * @param symbol
 *   The symbol string prepended to the first parameter.
 *   This is usually f_console_symbol_long_normal_s.
 * @param name
 *   The parameter name.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_parameter_cannot_use_with_pipe_
  extern f_status_t fll_program_print_error_parameter_cannot_use_with_pipe(fl_print_t * const print, const f_string_static_t symbol, const f_string_static_t name);
#endif // _di_fll_program_print_error_parameter_cannot_use_with_pipe_

/**
 * Print message about two parameters not being allowed to be used together with another specified parameter.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 * @param symbol_1
 *   The symbol string prepended to the first parameter.
 *   This is usually f_console_symbol_long_normal_s.
 * @param symbol_2
 *   The symbol string prepended to the second parameter.
 *   This is usually f_console_symbol_long_normal_s.
 * @param symbol_3
 *   The symbol string prepended to the third parameter.
 *   This is usually f_console_symbol_long_normal_s.
 * @param name_1
 *   The first parameter name.
 *
 *   This represents the parameter that imposes the exclusive or relation on the second and third parameters.
 * @param name_2
 *   The second parameter name.
 *
 *   This represents the parameter that is an exclusive or with the third parameter.
 * @param name_3
 *   The second parameter name.
 *
 *   This represents the parameter that is an exclusive or with the second parameter.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_parameter_allows_either_xor_
  extern f_status_t fll_program_print_error_parameter_cannot_use_with_xor(fl_print_t * const print, const f_string_static_t symbol_1, const f_string_static_t symbol_2, const f_string_static_t symbol_3, const f_string_static_t name_1, const f_string_static_t name_2, const f_string_static_t name_3);
#endif // _di_fll_program_print_error_parameter_allows_either_xor_

/**
 * Print message about parameter the parameter value being an empty string.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 * @param symbol
 *   The symbol string prepended to the parameter.
 *   This locks, uses, and unlocks the file stream.
 *   This is usually f_console_symbol_long_normal_s.
 * @param name
 *   The parameter name.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_parameter_empty_value_
  extern f_status_t fll_program_print_error_parameter_empty_value(fl_print_t * const print, const f_string_static_t symbol, const f_string_static_t name);
#endif // _di_fll_program_print_error_parameter_empty_value_

/**
 * Print message about parameter not being less than the given number.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 * @param symbol
 *   The symbol string prepended to the parameter.
 *   This locks, uses, and unlocks the file stream.
 *   This is usually f_console_symbol_long_normal_s.
 * @param name
 *   The parameter name.
 * @param value
 *   The value.
 * @param than
 *   The string representing the number to not be less than.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_parameter_integer_less_than_
  extern f_status_t fll_program_print_error_parameter_integer_less_than(fl_print_t * const print, const f_string_static_t symbol, const f_string_static_t name, const f_string_static_t value, const f_string_static_t than);
#endif // _di_fll_program_print_error_parameter_integer_less_than_

/**
 * Print message about parameter not being less than the given number.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 * @param symbol
 *   The symbol string prepended to the parameter.
 *   This locks, uses, and unlocks the file stream.
 *   This is usually f_console_symbol_long_normal_s.
 * @param name
 *   The parameter name.
 * @param value
 *   The value.
 * @param than
 *   The string representing the number to not be less than.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_parameter_integer_more_than_
  extern f_status_t fll_program_print_error_parameter_integer_more_than(fl_print_t * const print, const f_string_static_t symbol, const f_string_static_t name, const f_string_static_t value, const f_string_static_t than);
#endif // _di_fll_program_print_error_parameter_integer_more_than_

/**
 * Print message about parameter not being a valid integer.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 * @param symbol
 *   The symbol string prepended to the parameter.
 *   This locks, uses, and unlocks the file stream.
 *   This is usually f_console_symbol_long_normal_s.
 * @param name
 *   The parameter name.
 * @param value
 *   The value.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_parameter_integer_not_
  extern f_status_t fll_program_print_error_parameter_integer_not(fl_print_t * const print, const f_string_static_t symbol, const f_string_static_t name, const f_string_static_t value);
#endif // _di_fll_program_print_error_parameter_integer_not_

/**
 * Print message about parameter not being a negative integer.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 * @param symbol
 *   The symbol string prepended to the parameter.
 *   This locks, uses, and unlocks the file stream.
 *   This is usually f_console_symbol_long_normal_s.
 * @param name
 *   The parameter name.
 * @param value
 *   The value.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_parameter_integer_not_negative_
  extern f_status_t fll_program_print_error_parameter_integer_not_negative(fl_print_t * const print, const f_string_static_t symbol, const f_string_static_t name, const f_string_static_t value);
#endif // _di_fll_program_print_error_parameter_integer_not_negative_

/**
 * Print message about parameter not being a positive integer.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 * @param symbol
 *   The symbol string prepended to the parameter.
 *   This locks, uses, and unlocks the file stream.
 *   This is usually f_console_symbol_long_normal_s.
 * @param name
 *   The parameter name.
 * @param value
 *   The value.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_parameter_integer_not_positive_
  extern f_status_t fll_program_print_error_parameter_integer_not_positive(fl_print_t * const print, const f_string_static_t symbol, const f_string_static_t name, const f_string_static_t value);
#endif // _di_fll_program_print_error_parameter_integer_not_positive_

/**
 * Print message about parameter not have the required value associated with this parameter.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 * @param symbol
 *   The symbol string prepended to the parameter.
 *   This locks, uses, and unlocks the file stream.
 *   This is usually f_console_symbol_long_normal_s.
 * @param name
 *   The parameter name.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_parameter_missing_value_
  extern f_status_t fll_program_print_error_parameter_missing_value(fl_print_t * const print, const f_string_static_t symbol, const f_string_static_t name);
#endif // _di_fll_program_print_error_parameter_missing_value_

/**
 * Print message about parameter not have the required amount of values associated with this parameter.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 * @param symbol
 *   The symbol string prepended to the parameter.
 *   This locks, uses, and unlocks the file stream.
 *   This is usually f_console_symbol_long_normal_s.
 * @param name
 *   The parameter name.
 * @param amount
 *   A string representing the amount of missing parameters.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_parameter_missing_value_requires_amount_
  extern f_status_t fll_program_print_error_parameter_missing_value_requires_amount(fl_print_t * const print, const f_string_static_t symbol, const f_string_static_t name, const f_string_static_t amount);
#endif // _di_fll_program_print_error_parameter_missing_value_requires_amount_

/**
 * Print message about parameter only being allowed to be used once.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 * @param symbol
 *   The symbol string prepended to the parameter.
 *   This locks, uses, and unlocks the file stream.
 *   This is usually f_console_symbol_long_normal_s.
 * @param name
 *   The parameter name.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_parameter_must_specify_once_
  extern f_status_t fll_program_print_error_parameter_must_specify_once(fl_print_t * const print, const f_string_static_t symbol, const f_string_static_t name);
#endif // _di_fll_program_print_error_parameter_must_specify_once_

/**
 * Print message about a specific value only being allowed to be used once for the parameter.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 * @param symbol
 *   The symbol string prepended to the parameter.
 *   This locks, uses, and unlocks the file stream.
 *   This is usually f_console_symbol_long_normal_s.
 * @param name
 *   The parameter name.
 * @param value
 *   The value.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_parameter_must_specify_once_value_
  extern f_status_t fll_program_print_error_parameter_must_specify_once_value(fl_print_t * const print, const f_string_static_t symbol, const f_string_static_t name, const f_string_static_t value);
#endif // _di_fll_program_print_error_parameter_must_specify_once_value_

/**
 * Print message about parameter processing failure.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 * @param symbol
 *   The symbol string prepended to the parameter.
 *   This locks, uses, and unlocks the file stream.
 *   This is usually f_console_symbol_long_normal_s.
 * @param name
 *   The parameter name.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_parameter_process_
  extern f_status_t fll_program_print_error_parameter_process(fl_print_t * const print, const f_string_static_t symbol, const f_string_static_t name);
#endif // _di_fll_program_print_error_parameter_process_

/**
 * Print message about a specific value only being allowed to be used once for the parameter.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 * @param symbol
 *   The symbol string prepended to the parameter.
 *   This locks, uses, and unlocks the file stream.
 *   This is usually f_console_symbol_long_normal_s.
 * @param name
 *   The parameter name.
 * @param value_start
 *   The range start value.
 * @param value_stop
 *   The range stop value.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_parameter_range_start_before_stop_
  extern f_status_t fll_program_print_error_parameter_range_start_before_stop(fl_print_t * const print, const f_string_static_t symbol, const f_string_static_t name, const f_string_static_t value_start, const f_string_static_t value_stop);
#endif // _di_fll_program_print_error_parameter_range_start_before_stop_

/**
 * Print error message for when the parameter value is too long.
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 * @param symbol
 *   The symbol string prepended to the parameter.
 *   This locks, uses, and unlocks the file stream.
 *   This is usually f_console_symbol_long_normal_s.
 * @param name
 *   The parameter name.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_parameter_value_too_long_
  extern f_status_t fll_program_print_error_parameter_value_too_long(fl_print_t * const print, const f_string_static_t symbol, const f_string_static_t name);
#endif // _di_fll_program_print_error_parameter_value_too_long_

/**
 * Print a message about the pipe having an invalid form-feed character '\f' (U+000C).
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_pipe_invalid_form_feed_
  extern f_status_t fll_program_print_error_pipe_invalid_form_feed(fl_print_t * const print);
#endif // _di_fll_program_print_error_pipe_invalid_form_feed_

/**
 * Print a message about the pipe not having any content.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 */
#ifndef _di_fll_program_print_error_pipe_missing_content_
  extern f_status_t fll_program_print_error_pipe_missing_content(fl_print_t * const print);
#endif // _di_fll_program_print_error_pipe_missing_content_

/**
 * Print a message about the pipe having an Object without Content.
 *
 * This generally refers to an FSS Object and an FSS Content.
 *
 * This is only printed when verbosity is not set to quiet.
 *
 * This uses the following:
 *   - print.set->error: For the error context.
 *   - print.set->strong: For the highlighting context
 *   - print.prefix: For the prefixing a string to the message (such as "ERROR:").
 *
 * @param print
 *   The output structure.
 *   This locks, uses, and unlocks the file stream.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 *
 * @see f_file_stream_lock()
 * @see f_file_stream_unlock()
 * @see fl_print_format()
 */
#ifndef _di_fll_program_print_error_pipe_object_without_content_
  extern f_status_t fll_program_print_error_pipe_object_without_content(fl_print_t * const print);
#endif // _di_fll_program_print_error_pipe_object_without_content_

/**
 * Print standard help header.
 *
 * This print function does not use locking, be sure something like flockfile() and funlockfile() are appropriately called.
 *
 * @param print
 *   The output structure to print to.
 *   This uses but does not lock or unlock file stream.
 *   This requires print.set to be non-NULL.
 * @param name
 *   The name of the program.
 * @param version
 *   The version number of the program.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 *
 * @see fl_print_format()
 */
#ifndef _di_fll_program_print_help_header_
  extern f_status_t fll_program_print_help_header(fl_print_t * const print, const f_string_static_t name, const f_string_static_t version);
#endif // _di_fll_program_print_help_header_

/**
 * Print standard help operations.
 *
 * This print function does not use locking, be sure something like flockfile() and funlockfile() are appropriately called.
 *
 * @param print
 *   The output structure to print to.
 *   This uses but does not lock or unlock file stream.
 *   This requires print.set to be non-NULL.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 *
 * @see fl_print_format()
 */
#ifndef _di_fll_program_print_help_operations_
  extern f_status_t fll_program_print_help_operations(fl_print_t * const print);
#endif // _di_fll_program_print_help_operations_

/**
 * Print standard help option.
 *
 * This print function does not use locking, be sure something like flockfile() and funlockfile() are appropriately called.
 *
 * @param print
 *   The output structure to print to.
 *   This uses but does not lock or unlock file stream.
 *   This requires print.set to be non-NULL.
 * @param option_short
 *   The short name of the option.
 * @param option_long
 *   The long name of the option.
 * @param symbol_short
 *   The short symbol of the option.
 * @param symbol_long
 *   The long symbol of the option.
 * @param description
 *   A desciption associated with the option.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 *
 * @see fl_print_format()
 */
#ifndef _di_fll_program_print_help_option_
  extern f_status_t fll_program_print_help_option(fl_print_t * const print, const f_string_static_t option_short, const f_string_static_t option_long, const f_string_static_t symbol_short, const f_string_static_t symbol_long, const char * const description);
#endif // _di_fll_program_print_help_option_

/**
 * Print standard help option (long option only).
 *
 * This print function does not use locking, be sure something like flockfile() and funlockfile() are appropriately called.
 *
 * @param print
 *   The output structure to print to.
 *   This uses but does not lock or unlock file stream.
 *   This requires print.set to be non-NULL.
 * @param option_long
 *   The long name of the option.
 * @param symbol_long
 *   The long symbol of the option.
 * @param description
 *   A desciption associated with the option.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 *
 * @see fl_print_format()
 */
#ifndef _di_fll_program_print_help_option_long_
  extern f_status_t fll_program_print_help_option_long(fl_print_t * const print, const f_string_static_t option_long, const f_string_static_t symbol_long, const char * const description);
#endif // _di_fll_program_print_help_option_long_

/**
 * Print standard help option (other option only).
 *
 * This print function does not use locking, be sure something like flockfile() and funlockfile() are appropriately called.
 *
 * @param print
 *   The output structure to print to.
 *   This uses but does not lock or unlock file stream.
 *   This requires print.set to be non-NULL.
 * @param option_other
 *   The other name of the option.
 * @param description
 *   A desciption associated with the option.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 *
 * @see fl_print_format()
 */
#ifndef _di_fll_program_print_help_option_other_
  extern f_status_t fll_program_print_help_option_other(fl_print_t * const print, const f_string_static_t option_other, const char * const description);
#endif // _di_fll_program_print_help_option_other_

/**
 * Print standard help option (short option only).
 *
 * This print function does not use locking, be sure something like flockfile() and funlockfile() are appropriately called.
 *
 * @param print
 *   The output structure to print to.
 *   This uses but does not lock or unlock file stream.
 *   This requires print.set to be non-NULL.
 * @param option_short
 *   The short name of the option.
 * @param symbol_short
 *   The short symbol of the option.
 * @param description
 *   A desciption associated with the option.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 *
 * @see fl_print_format()
 */
#ifndef _di_fll_program_print_help_option_short_
  extern f_status_t fll_program_print_help_option_short(fl_print_t * const print, const f_string_static_t option_short, const f_string_static_t symbol_short, const char * const description);
#endif // _di_fll_program_print_help_option_short_

/**
 * Print all standard help options.
 *
 * This print function does not use locking, be sure something like flockfile() and funlockfile() are appropriately called.
 *
 * @param print
 *   The output structure to print to
 *   This uses but does not lock or unlock file stream.
 *   This requires print.set to be non-NULL.
 * @param context
 *   The color context.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 *
 * @see fl_print_format()
 */
#ifndef _di_fll_program_print_help_option_standard_
  extern f_status_t fll_program_print_help_option_standard(fl_print_t * const print);
#endif // _di_fll_program_print_help_option_standard_

/**
 * Print standard help special options.
 *
 * This print function does not use locking, be sure something like flockfile() and funlockfile() are appropriately called.
 *
 * @param print
 *   The output structure to print to.
 *   This uses but does not lock or unlock file stream.
 *   This requires print.set to be non-NULL.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 *
 * @see fl_print_format()
 */
#ifndef _di_fll_program_print_help_special_options_
  extern f_status_t fll_program_print_help_special_options(fl_print_t * const print);
#endif // _di_fll_program_print_help_special_options_

/**
 * Print standard help usage.
 *
 * This print function does not use locking, be sure something like flockfile() and funlockfile() are appropriately called.
 *
 * @param print
 *   The output structure to print to.
 *   This uses but does not lock or unlock file stream.
 *   This requires print.set to be non-NULL.
 * @param name
 *   The name of the program.
 * @param parameters
 *   (optional) The non-option parameters to be displayed inside the brackets.
 *   Set the first array value to EOS to disable printing of parameters.
 *   Set parameters.used to 0 to disable.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 *
 * @see f_print_dynamic_raw()
 * @see fl_print_format()
 */
#ifndef _di_fll_program_print_help_usage_
  extern f_status_t fll_program_print_help_usage(fl_print_t * const print, const f_string_static_t name, const f_string_static_t parameters);
#endif // _di_fll_program_print_help_usage_

/**
 * Print a message about a process signal being recieved, such as an interrupt signal, as a warning.
 *
 * This is only printed when verbosity is set to verbose or debug.
 *
 * @param print
 *   The output structure to print to.
 *   This locks, uses, and unlocks the file stream.
 *   This requires print.set to be non-NULL.
 * @param signal
 *   The signal code received.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 *
 * @see f_file_stream_lock()
 * @see f_file_stream_unlock()
 * @see fl_print_format()
 */
#ifndef _di_fll_program_print_signal_received_
  extern f_status_t fll_program_print_signal_received(fl_print_t * const print, const uint32_t signal);
#endif // _di_fll_program_print_signal_received_

/**
 * Print the program version.
 *
 * @param print
 *   The output structure to print to.
 *   This uses but does not lock or unlock file stream.
 *   This requires print.set to be non-NULL.
 * @param version
 *   The version number of the program.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_parameter (with error bit) if a parameter is invalid.
 *
 * @see f_file_stream_lock()
 * @see f_file_stream_unlock()
 * @see f_print_dynamic()
 * @see f_print_dynamic_raw()
 */
#ifndef _di_fll_program_print_version_
  extern f_status_t fll_program_print_version(fl_print_t * const print, const f_string_static_t version);
#endif // _di_fll_program_print_version_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _FLL_program_print_h
